/*
 *  This file if part of nzbget
 *
 *  Copyright (C) 2004  Sven Henkel <sidddy@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Revision: 1.9 $
 * $Date: 2005/07/12 09:37:48 $
 *
 */


#ifndef CONNECTION_H
#define CONNECTION_H

#include <sys/socket.h>
#include <sys/types.h>
#include <stdio.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "NetAddress.h"

#define CONNECTION_STATUS_CONNECTED		1
#define CONNECTION_STATUS_DISCONNECTED	2
#define CONNECTION_STATUS_LISTENING		3
#define CONNECTION_BUFFER_SIZE			1024*10 // Allocate 10Kb for the buffer

class Connection
{
protected:
	struct sockaddr_in	m_SocketAddress;
	NetAddress* 		m_pNetAddress;
	int					m_iSocket;
	int					m_iStatus;
	FILE*				m_pFileBuffer;
	int					m_iBufferLength;
	int					m_iBufferPos;
	char* 				m_pReadBuffer;
	float				m_fDownloadTimeout;
public:

						Connection(NetAddress* pNetAddress);
	virtual 			~Connection();
	int 				Connect();
	int					Disconnect();
	int					Bind();
	int					Write(char* text);
	int					Send( char* pData, int iLength );
	int					Send( int iSocket, char* pData, int iLength );
	char*				ReadLine();
	int					Accept();
	int					Recv( int iSocket );
	int					Recv( int iSocket, char* pBuffer, int iSize );
	NetAddress*			GetServer() { return m_pNetAddress; }
	int					GetSocket() { return m_iSocket; }
	struct sockaddr_in* GetSocketAddressStruct() { return &m_SocketAddress; }
	const char*			GetReadBuffer() const { return m_pReadBuffer; }
	int 				SendBuffer( void* pData, int iLength );
	int 				SendBuffer( int iSocket, void* pData, int iLength );
	void				FlushFileBuffer();
			
	// Helper functions
	const char*			GetStatusString();

protected:
    virtual int 		DoConnect();
    virtual int			DoDisconnect();
	virtual int			DoBind();
    virtual int			DoWrite(char* text);
    virtual char*		DoReadLine();
	virtual int			DoAccept();
};

#endif
