/*
 *  This file if part of nzbget
 *
 *  Copyright (C) 2004  Sven Henkel <sidddy@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Revision: 1.19 $
 * $Date: 2005/07/12 09:37:48 $
 *
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <unistd.h>

#include "global.h"
#include "Coordinator.h"
#include "Job.h"
#include "DownloadJob.h"
#include "MultiServerPool.h"
#include "Frontend.h"

extern Frontend* g_pFrontend;

Coordinator::Coordinator(NZBQueue* pQueue, Options* pOptions)
{
	m_pQueue			= pQueue;
	m_pOptions			= pOptions;
	m_bDownloadCanceled	= false;
}

Coordinator::~Coordinator()
{
}

void Coordinator::Start()
{
	m_FileDownloads.clear();

	while( !m_bDownloadCanceled )
	{
		Job* pJob = m_pQueue->GetNextJob();
      
		if( !pJob )
		{
			break;
		}

		g_pFrontend->SetQueueSize( m_pQueue->GetWaitingSize() );
      
		if( pJob->GetType() == JOBTYPE_DOWNLOAD )
		{
			char szDestinationDir[1024];
			
			GetDestinationDir( szDestinationDir );		
			
			FileDownloader* pFileDownloader = new FileDownloader( (DownloadJob*)pJob, m_pOptions->getOption(OPTION_TEMPDIR,"."), szDestinationDir );
			
			m_FileDownloads.push_back( pFileDownloader );

			pFileDownloader->Start();
		}

		std::vector<FileDownloader*>::iterator it;
		it = m_FileDownloads.begin();
		while( it != m_FileDownloads.end() )
		{
			FileDownloader* pFileDownloader = *it;
			if( pFileDownloader->IsFinished() )
			{
				it = m_FileDownloads.erase( it );
				delete pFileDownloader;
			}
			else
			{
				it++;
			}
		};
	}

	// no more jobs!
	// wait for running ones to finish
	while( !m_bDownloadCanceled )
	{
		bool bStop = true;
	
		std::vector<FileDownloader*>::iterator it;
		it = m_FileDownloads.begin();
		while( it != m_FileDownloads.end() )
		{
			FileDownloader* pFileDownloader = *it;
			if( pFileDownloader->IsRunning() )
			{
				bStop = false;
				it++;
			}
			else
			{
				it = m_FileDownloads.erase( it );
				delete pFileDownloader;
			}
		}

		if( bStop )
		{
			break;
		}
      
		usleep(1000*1000);
    }
}

void Coordinator::GetDestinationDir( char* szBuffer )
{
	const char* szFileName = m_pOptions->getOption( OPTION_NZBFILE, NULL );
	
	if( !strcmp( m_pOptions->getOption( OPTION_APPENDDIR, NULL ), "yes" ) )
	{
		char postname[1024];
		const char* szBaseName = basename( szFileName );
		
		// if .nzb file has a certain structure, try to strip out certain elements
		if (sscanf(szBaseName,"msgid_%*d_%1023s", postname) == 1)
		{
			char* tmp1;
			char* tmp2;
			
			// wipe out ".nzb"
			memset(rindex(postname, '.'), 0, postname+strlen(postname)-rindex(postname, '.'));
			
			tmp1 = m_pOptions->getOption(OPTION_REALDESTDIR, ".");
			int slen = strlen(tmp1) + 1 + strlen(postname) + 3;
			tmp2 = (char*) malloc(slen);
			memset(tmp2, 0, slen);
			if (tmp1[strlen(tmp1) - 1] != '/')
			{
				strcpy(tmp2, tmp1);
				strcat(tmp2, "/");
				strcat(tmp2, postname);
			}
			else
			{
				strcpy(tmp2, tmp1);
				strcat(tmp2, postname);
			}
			strcpy( szBuffer, tmp2);
		}
		else
		{
			// Just use the name of the .nzb file as a directory
			strcpy( szBuffer, szBaseName );
		}
	}
	else
	{
		strcpy( szBuffer, m_pOptions->getOption(OPTION_REALDESTDIR, ".") );
	}
}

void Coordinator::CancelDownload()
{
	m_bDownloadCanceled = true;

	std::vector<FileDownloader*>::iterator it;
	for( it=m_FileDownloads.begin(); it != m_FileDownloads.end(); it++ )
	{
		(*it)->CancelDownload();
	}
}
