/*
 *  This file if part of nzbget
 *
 *  Copyright (C) 2004  Sven Henkel <sidddy@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Revision: 1.12 $
 * $Date: 2005/07/06 18:34:34 $
 *
 */


#ifndef FILEDOWNLOADER_H
#define FILEDOWNLOADER_H

#include "DownloadJob.h"
#include "ArticleJob.h"
#include "ArticleDownloader.h"
#include "Observable.h"
#include "Observer.h"

class FileDownloader : public Observer
{
	DownloadJob*					m_pJob;
	std::vector<ArticleJob*>		m_ArticleJobs;
	std::vector<ArticleDownloader*>	m_ArticleDownloads;
	char* 							m_szTempDir;
	char* 							m_szDestDir;
	bool							m_bCancelDownload;
	float							m_fDownloadTimeout;

	pthread_mutex_t					m_ArticleDownloadMutex;
public:
    								FileDownloader(DownloadJob* job, char* tempdir, char* destdir);
    virtual 						~FileDownloader();

    virtual void 					Notify(Observable* caller);

    void 							Start();

    bool 							IsRunning();
	bool 							IsFinished() { return !IsRunning();}
	bool 							IsStopping() { return m_bCancelDownload; }
	void 							CancelDownload();
	void							CleanupArticleDownloads();
	void							PlotArticleDownloads();
	void							LogBrokenFile( const char* szBrokenLogName, const char* szFileName, int iParts, int iBroken );
 protected:
    bool 							IsDupe();
};

#endif
