/*
 *  This file if part of nzbget
 *
 *  Copyright (C) 2004  Sven Henkel <sidddy@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Revision: 1.15 $
 * $Date: 2005/07/12 09:37:48 $
 *
 */

#ifndef FRONTEND_H
#define FRONTEND_H

#include <vector>
#include <pthread.h>
#include <time.h>

#include "Thread.h"
#include "ArticleJob.h"
#include "Connection.h"

class DownloadSpeedMonitor;

class Frontend : public Thread
{
protected:
	DownloadSpeedMonitor*	m_pDownloadSpeedMonitor;
	std::vector< char* >	m_Messages;
	pthread_mutex_t			m_MutexMessages;
	long long 				m_lQueueSize;
public:

    						Frontend();
    virtual 				~Frontend();

    virtual void 			FrontendError(char* msg);
    virtual void 			FrontendWarn(char* msg);
    virtual void 			FrontendDebug(char* msg);
    virtual void 			FrontendInfo(char* msg);
    virtual void 			SetQueueSize(long long s) { m_lQueueSize = s; }
    virtual void 			PrintMessages();
    virtual void 			PrintStatus();
    virtual void 			FrontendQueueListEntry( const char* szEntry );
	virtual int				GetSleepTime() { return 500 * 1000; }
	virtual void			OnDownloadCompleted( const char* szFileName ) {};
	virtual void        	Refresh() {}; // empty function - override if needed
	virtual void			Start();
	virtual void			Stop();
protected:
	virtual void 			Run();
};

//--------------------------------------------------------------------------------------------
//////////////////////////////////////////////////////////////////////////////////////////////
//--------------------------------------------------------------------------------------------

class DownloadSpeedMonitor : public Thread
{
	std::vector< float >	m_Seconds10;
	std::vector< float >	m_Minutes;
	std::vector< float >	m_Minutes10;
	std::vector< float >	m_Hours;
	
public:
							DownloadSpeedMonitor();
	virtual void			Run();
	virtual int				GetSleeptime() const { return 10 * 1000 * 1000; }
	virtual void			UpdateDownloadSpeed();
private:
	void					ClearStats();
};

#endif
