/*
 *  This file if part of nzbget
 *
 *  Copyright (C) 2005  Bo Cordes Petersen <placebodk@sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Revision: 1.10 $
 * $Date: 2005/07/12 09:37:48 $
 *
 */

#include "NZBQueueMessageControl.h"
#include "NZBClient.h"
#include "Options.h"
#include "Frontend.h"
#include "unistd.h"

extern Options* g_pOptions;

extern bool CreateNZBCharBufferFromFilename( const char* szFileName, char** pBuffer, int* pBufferLength );

NZBClient::NZBClient()
{
	m_pConnection	= NULL;
	m_pNetAddress	= NULL;
}

NZBClient::~NZBClient()
{
	if( m_pConnection )
	{
		delete m_pConnection;
	}

	if( m_pNetAddress )
	{
		delete m_pNetAddress;
	}
}

bool NZBClient::InitConnection()
{
	if( m_pConnection ) return true;
	
	// Create a connection to the server
	m_szNetAddress	= g_pOptions->getOption( OPTION_SERVERIP, NULL );
	m_iPort			= atoi( g_pOptions->getOption( OPTION_SERVERPORT, NULL ) );
	m_pNetAddress	= new NetAddress( m_szNetAddress, m_iPort );
	m_pConnection	= new Connection( m_pNetAddress );

	return true;
}

/*
 * Sends a message to the running nzbget process.
 */
bool NZBClient::RequestServerDownloadFile( const char* szName )
{
	if( !InitConnection() ) return false;
	
	const char* szPassword		= g_pOptions->getOption( OPTION_SERVERPASSWORD, NULL );
	
	// Read the file into the buffer
	char* szBuffer	= NULL;
	int iLength		= 0;
	CreateNZBCharBufferFromFilename( szName, &szBuffer, &iLength );

	if( m_pConnection->Connect() >= 0 )
	{
		// Send a "new download" request
		SNZBQueueDownloadRequest DownloadRequest;

		InitMessageBase( &DownloadRequest.m_MessageBase, NZBMessageRequest::eRequestDownload, sizeof( DownloadRequest ) );
		DownloadRequest.m_iTrailingDataLength = iLength;

		printf( "m_iTrailingDataLength %i\n", DownloadRequest.m_iTrailingDataLength );

		if( strlen( szName ) < NZBREQUESTFILENAMESIZE )
		{
			strcpy( DownloadRequest.m_szFileName, szName );
		}
		else
		{
			memcpy( DownloadRequest.m_szFileName, szName, NZBREQUESTFILENAMESIZE-1 );
			DownloadRequest.m_szFileName[ NZBREQUESTFILENAMESIZE-1 ] = 0;
		}

		strncpy( DownloadRequest.m_MessageBase.m_szPassword, szPassword, NZBREQUESTPASSWORDSIZE-1 );

		if( m_pConnection->Send( (char*) (&DownloadRequest), sizeof( DownloadRequest )  ) < 0 )
		{
			perror( "m_pConnection->Write" );
			return false;
		}
		else
		{
#ifdef NZBGET_TEST_SEND_RECEIVE
			// TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST
			// TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST 
			FILE* pFile = fopen( "sending.txt", "a+" );

			if( !pFile )
			{
				perror( "fopen" );
			}
			else
			{
				fprintf( pFile, "%s", szBuffer );
				fclose( pFile );
			}
			// TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST
			// TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST 
#endif
			m_pConnection->Write( szBuffer );
			m_pConnection->FlushFileBuffer();

			char szAnswer[1024];
			strcpy( szAnswer, "N/A" );
			m_pConnection->Recv( m_pConnection->GetSocket(), szAnswer, 1024 );
			printf( "nzbserver returned: %s\n", szAnswer );
			
			m_pConnection->Disconnect();
		}
	}
	else
	{
		printf( "Unable to send request to nzbserver at %s (port %i)\n", m_szNetAddress, m_iPort );
	}

	// Cleanup
	if( szBuffer )
	{
		free( szBuffer );
	}

	return true;
}

bool NZBClient::RequestServerDownloadList()
{
	if( !InitConnection() ) return false;
	
	if( m_pConnection->Connect() < 0 ) return false;
	
	// Send a "new download" request
	SNZBMessageBase ListRequest;

	InitMessageBase( &ListRequest, NZBMessageRequest::eRequestList, sizeof( ListRequest ) );

	if( m_pConnection->Send( (char*) (&ListRequest), sizeof( ListRequest )  ) < 0 )
	{
		perror( "m_pConnection->Write" );
		return false;
	}

	// Now listen for the returned list
	SNZBQueueListRequestAnswer ListRequestAnswer;
	if( m_pConnection->Recv( m_pConnection->GetSocket(), (char*) &ListRequestAnswer, sizeof( ListRequestAnswer ) ) < 0 )
	{
		return false;
	}

	if( ListRequestAnswer.m_iNrTrailingEntries == 0 )
	{
		printf( "Server has no files queued for download\n" );
	}
	else
	{
		SNZBQueueListRequestAnswerEntry* pListAnswer = new SNZBQueueListRequestAnswerEntry[ ListRequestAnswer.m_iNrTrailingEntries ];
	
		if( m_pConnection->Recv( m_pConnection->GetSocket(), (char*) pListAnswer, sizeof( SNZBQueueListRequestAnswerEntry ) * ListRequestAnswer.m_iNrTrailingEntries ) < 0 )
		{
			delete pListAnswer;
			return false;
		}
	
		// Printout the queue
	
		printf( "-----------------------------------\n" );
		printf( "Queue List\n" );
		printf( "-----------------------------------\n" );
	
		int i;
		for( i=0; i<ListRequestAnswer.m_iNrTrailingEntries; i++ )
		{
			printf( "%i. Filename %s (%i/%i)\n", pListAnswer[i].m_iEntry, pListAnswer[i].m_szFileName, pListAnswer[i].m_iNumberDownloads, pListAnswer[i].m_iOriginalNumberDownloads );
		}

		// Print out the download rate
		printf( "\nDownload rate %f\n", ListRequestAnswer.m_fDownloadRate );
	
		delete pListAnswer;
	}
	
	m_pConnection->Disconnect();

	return true;
}

bool NZBClient::RequestServerDownloadPauseUnpause( bool bPause )
{

	if( !InitConnection() ) return false;
	
	if( m_pConnection->Connect() < 0 ) return false;

	// Send a "new download" request
	SNZBQueuePauseUnpauseRequest PauseUnpauseRequest;

	InitMessageBase( &PauseUnpauseRequest.m_MessageBase, NZBMessageRequest::eRequestPauseUnpause, sizeof( PauseUnpauseRequest ) );
	PauseUnpauseRequest.m_bPause = bPause;

	if( m_pConnection->Send( (char*) (&PauseUnpauseRequest), sizeof( PauseUnpauseRequest )  ) < 0 )
	{
		perror( "m_pConnection->Send" );
		m_pConnection->Disconnect();
		return false;
	}

	m_pConnection->Disconnect();

	return true;
}

bool NZBClient::RequestServerSetDownloadRate( float fRate )
{
	if( !InitConnection() ) return false;
	
	if( m_pConnection->Connect() < 0 ) return false;

	// Send a "new download" request
	SNZBQueueSetDownloadRate SetDownloadRateRequest;

	InitMessageBase( &SetDownloadRateRequest.m_MessageBase, NZBMessageRequest::eRequestSetDownloadRate, sizeof( SetDownloadRateRequest ) );
	SetDownloadRateRequest.m_fDownloadRate = fRate;

	if( m_pConnection->Send( (char*) (&SetDownloadRateRequest), sizeof( SetDownloadRateRequest )  ) < 0 )
	{
		perror( "m_pConnection->Send" );
		m_pConnection->Disconnect();

		return false;
	}

	m_pConnection->Disconnect();

	return true;
}

void NZBClient::InitMessageBase( SNZBMessageBase* pMessageBase, int iRequest, int iSize )
{
	const char* szPassword		= g_pOptions->getOption( OPTION_SERVERPASSWORD, NULL );

	pMessageBase->m_iId	= 'nzbg';
	pMessageBase->m_iType = iRequest;
	pMessageBase->m_iSize = iSize;
	
	strncpy( pMessageBase->m_szPassword, szPassword, NZBREQUESTPASSWORDSIZE-1 );
}
