/*
 *  This file if part of nzbget
 *
 *  Copyright (C) 2005  Bo Cordes Petersen <placebodk@sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Revision: 1.5 $
 * $Date: 2005/07/12 09:37:48 $
 *
 */

#include "NZBClientServerMessageBase.h"
#include "NZBQueueMessageControl.h"

#include "Options.h"

extern Options* g_pOptions;

bool CreateNZBCharBufferFromFilename( const char* szFileName, char** pBuffer, int* pBufferLength );

/*
 *
 */

const char* g_szMessageRequests[] = { "N/A", "Download", "Pause/Unpause", "List", "N/A", "Set download rate" };

/*
 * Initializes a message base, filling out the member variables with iRequest, iSize and Id and password
 */
void NZBClientServerMessageBase::InitMessageBase( SNZBMessageBase* pMessageBase, int iRequest, int iSize )
{
	const char* szPassword		= g_pOptions->getOption( OPTION_SERVERPASSWORD, NULL );

	pMessageBase->m_iId	= 'nzbg';
	pMessageBase->m_iType = iRequest;
	pMessageBase->m_iSize = iSize;
	
	strncpy( pMessageBase->m_szPassword, szPassword, NZBREQUESTPASSWORDSIZE-1 );
}

/*
 * Checks whether data casted to SNZBMessageBase has the 
 */
bool NZBClientServerMessageBase::CheckDataIsNZBMessage( SNZBMessageBase* pMessage )
{
	return pMessage->m_iId == 'nzbg';
}

bool CreateNZBCharBufferFromFilename( const char* szFileName, char** pBuffer, int* pBufferLength )
{
	FILE* pFile = fopen( szFileName, "r" );

	if( pFile==NULL )
	{
		exit( -1 );
		return false;
	}

	// obtain file size.
	fseek (pFile , 0 , SEEK_END);
	int iSize  = ftell (pFile);
	rewind (pFile);

	// allocate memory to contain the whole file.
	*pBuffer = (char*) malloc ( iSize+1 );
	if( *pBuffer == NULL)
	{
		exit( -1 );
		return false;
	}
	
	// copy the file into the buffer.
	fread( *pBuffer, 1, iSize, pFile );

	fclose (pFile);

	(*pBuffer)[ iSize ] = 0;

	*pBufferLength = iSize;

	return true;
}
