/*
 *  This file if part of nzbget
 *
 *  Copyright (C) 2005  Bo Cordes Petersen <placebodk@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Revision: 1.4 $
 * $Date: 2005/06/27 21:31:31 $
 *
 */


#ifndef __NZBCLIENTSERVERMESSAGEBASE_H__
#define __NZBCLIENTSERVERMESSAGEBASE_H__

#include <sys/types.h>

#define NZBMESSAGEQUEUESIZE 	512

#define NZBREQUESTFILENAMESIZE	512

#define NZBREQUESTPASSWORDSIZE	32

//#define NZBGET_TEST_SEND_RECEIVE

#include <unistd.h>

namespace NZBMessageRequest
{
	enum
	{
		eRequestDownload = 1,
		eRequestPauseUnpause,
		eRequestList,
		eRequestListAnswer,
		eRequestSetDownloadRate
	};
}

// The basic NZBMessageBase struct
struct SNZBMessageBase
{
	int32_t					m_iId;		// Id must be 'nzbg' in integer-value
	int32_t					m_iType;	// message type, must be > 0
	int32_t					m_iSize;	// Size of the entire struct
	char					m_szPassword[ NZBREQUESTPASSWORDSIZE ];	// Password needs to be in every request
};

// A download request
struct SNZBQueueDownloadRequest
{
	SNZBMessageBase			m_MessageBase;	// Must be the first in the struct
	char					m_szFileName[ NZBREQUESTFILENAMESIZE ];
	int32_t					m_iTrailingDataLength;
};

// A list request-answer
struct SNZBQueueListRequestAnswer
{
	SNZBMessageBase			m_MessageBase;			// Must be the first in the struct
	float					m_fDownloadRate;
	int32_t					m_iNrTrailingEntries;	//
};

// A list request-answer
struct SNZBQueueListRequestAnswerEntry
{
	char					m_szFileName[ NZBREQUESTFILENAMESIZE ];
	int32_t					m_iOriginalNumberDownloads;
	int32_t					m_iNumberDownloads;
	int32_t					m_iEntry;
};

// A Pause/Unpause request
struct SNZBQueuePauseUnpauseRequest
{
	SNZBMessageBase			m_MessageBase;	// Must be the first in the struct
	bool					m_bPause;		// The value g_bPause should be set to
};

// Request setting the download rate
struct SNZBQueueSetDownloadRate
{
	SNZBMessageBase			m_MessageBase;		// Must be the first in the struct
	float					m_fDownloadRate;
};

class NZBClientServerMessageBase
{
protected:
	void				InitMessageBase( SNZBMessageBase* pMessageBase, int iRequest, int iSize );
	bool				CheckDataIsNZBMessage( SNZBMessageBase* pMessage );
};

#endif
