/*
 *  This file if part of nzbget
 *
 *  Copyright (C) 2004  Sven Henkel <sidddy@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Revision: 1.10 $
 * $Date: 2005/07/12 09:37:48 $
 *
 */


#ifndef NZBQUEUE_H
#define NZBQUEUE_H

#include <libxml/parser.h>
#include <vector>
#include "Job.h"

class NZBQueue
{
	std::vector<Job*> 	m_Jobs;     // Download jobs for the NZBQueue
	int					m_iOriginalNumberOfJobs;
	char*				m_szFileName;
	char*				m_szBaseFileName;
	bool				m_bPause;
	bool				m_bCancelDownload;
	bool				m_bExistInQueueDir;
public:
						NZBQueue( const char* szFileName, xmlDocPtr nzbdoc );
	virtual 			~NZBQueue();
	
	virtual Job* 		GetNextJob();
	virtual long long 	GetWaitingSize();
	const char* 		GetFileName() const { return m_szFileName; }
	const char*			GetBaseFileName() const { return m_szBaseFileName; }
	void 				PrintProgressInfo();
	int					GetOriginalNumberOfJobs() const { return m_iOriginalNumberOfJobs; }
	int					GetNumberOfJobs() const { return m_Jobs.size(); }
	bool				GetPause() const { return m_bPause; }
	void				SetPause( bool bOnOff ) { m_bPause = bOnOff; }
	void				TogglePause() { m_bPause = !m_bPause; }
	void				SetCancelDownload( bool bOnOff ) { m_bCancelDownload = bOnOff; }
	bool				GetCancelDownload() { return m_bCancelDownload; }
	bool				GetExistInQueueDir() const { return m_bExistInQueueDir; }
	void				SetExistInQueueDir( bool bOnOff ) { m_bExistInQueueDir = bOnOff; }

 protected:

    void 				parseNZB(xmlDocPtr nzb);
    void 				parseFile(xmlNodePtr segments);
    void 				parseSegment(xmlNodePtr node, std::vector<char*>* idlist, long long* size);
    void 				parseGroup(xmlNodePtr node, std::vector<char*>* grouplist);    
};

#endif
