/*
 *  This file if part of nzbget
 *
 *  Copyright (C) 2004  Sven Henkel <sidddy@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Revision: 1.5 $
 * $Date: 2005/07/12 09:37:48 $
 *
 */


#ifndef __NZBQUEUECOORDINATOR_H__
#define __NZBQUEUECOORDINATOR_H__

#include "deque"
#include "Thread.h"
#include "Coordinator.h"
#include "Options.h"
#include "NZBQueue.h"

typedef struct _RunningDL
{
	ArticleJob*		m_pJob;
	Connection*		m_pConnection;
	int				bytes;
	int				lines;
	int				maxlines;
	time_t			starttime;
	
	const char* GetJobName()
	{
		return m_pJob->GetMessageID();
	}
	
	const char* GetServerName()
	{
		return m_pConnection->GetServer()->GetHost();
	}
} SRunningDL;

class NZBQueueMessageControl;

class NZBQueueCoordinator
{
public:
    static pthread_mutex_t m_Mutex;
private:
	std::deque<NZBQueue*>		m_NZBQueueList;
	NZBQueue*					m_pCurrentNZBQueue;
	Options*					m_pOptions;
	NZBQueueMessageControl*		m_pQueueMessageControl;
	bool						m_bServerMode;
	bool						m_bSaveQueueList;;
	

	std::vector<SRunningDL*>	m_Running;
	pthread_mutex_t				m_mutexRunning;
	
	Coordinator* 				m_pCoordinator;
public:
    				NZBQueueCoordinator( Options* pOptions );
					~NZBQueueCoordinator();
    
	void			Start();
	void			AddNZBQueueToDownloadQueue( NZBQueue* pNZBQueue, bool bUseQueueDir );
	NZBQueue*		AddNZBFileToQueue( const char* szFileName, bool bUseQueueDir );
	NZBQueue*		CreateNZBQueueFromBuffer( const char* szFilename, const char* szBuffer, int iSize, bool bUseQueueDir = true );
	std::deque<NZBQueue*>&	GetNZBQueueList() { return m_NZBQueueList; }
	
	void			JobStarted( ArticleJob* pJob, Connection* pConnection );
	void			JobFinished( ArticleJob* pJob, Connection* pConnection );
	void			JobProgress( ArticleJob* pJob, Connection* pConnection, int bytes, int lines, int maxlines);

	float			CalcCurrentDownloadSpeed();
				
	void			PrintNZBQueueList();

	// Editing the queue
	bool			EditQueuePauseUnpauseEntry( uint iEntry );
	bool			EditQueueDeleteEntry( uint iEntry );
	bool			EditQueueMoveEntryUp( uint iEntry );
	bool			EditQueueMoveEntryDown( uint iEntry );

private:

	// Save & load of nzb-queues
	bool			SaveNZBFileToQueueDir( const char* szFileName, const char* szBuffer );
	bool			RemoveNZBFileFromDir( const char* szFileName );
	bool			SaveNZBQueueList();
	bool			LoadNZBQueueList( bool bParseFiles=true );
	bool			CreateQueueDir();
	bool			CleanupNZBQueueList();
		
};

#endif
