/*
 *  This file if part of nzbget
 *
 *  Copyright (C) 2005  Bo Cordes Petersen <placebodk@sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Revision: 1.17 $
 * $Date: 2005/07/13 15:06:03 $
 *
 */

#include "NZBQueueMessageControl.h"
#include "NZBQueueCoordinator.h"
#include "global.h"
#include "GlobalControl.h"
#include "Options.h"
#include "Frontend.h"
#include "unistd.h"
#include "NZBQueueCoordinator.h"

extern Frontend* g_pFrontend;
extern Options* g_pOptions;
extern NZBQueueCoordinator* g_pNZBQueueCoordinator;
extern const char* g_szMessageRequests[];

NZBQueueMessageControl::NZBQueueMessageControl()
{
	const char* szNetAddress = g_pOptions->getOption( OPTION_SERVERIP, NULL );
	m_iPort = atoi( g_pOptions->getOption( OPTION_SERVERPORT, NULL ) );
	m_szPassword = g_pOptions->getOption( OPTION_SERVERPASSWORD, NULL );
	m_pNetAddress = new NetAddress( szNetAddress, m_iPort );
	m_pConnection = new Connection( m_pNetAddress );

	m_pConnection->Bind();
}

NZBQueueMessageControl::~NZBQueueMessageControl()
{
}

void NZBQueueMessageControl::Run()
{
	int iResult = 0;

	while( true )
	{
		int iSocket;
		// Accept connections and store the "new" socket value
		if( (iSocket = m_pConnection->Accept()) == -1 )
		{
			continue;
		}

		int iRequestReceived = 0;

		// Read the first package which needs to be a request
		char RequestBuffer[8192];
		
		iRequestReceived = m_pConnection->Recv( iSocket, (char*)&RequestBuffer, sizeof( RequestBuffer )  );
		if( iRequestReceived < 0 )
		{
			continue;
		}

		SNZBMessageBase* pMessageBase = (SNZBMessageBase*) &RequestBuffer;

		// Make sure this is a nzbget request from a client
		if( !CheckDataIsNZBMessage( pMessageBase ) )
		{
			info( "Non-nzbget request received on port %i", m_iPort );
			nzblog( "Non-nzbget request received on port %i", m_iPort );

			if( iSocket > -1 )
			{
			  close( iSocket );
			}
			
			continue;
		}

		if( strcmp( pMessageBase->m_szPassword, m_szPassword ) )
		{
			info( "nzbget request received on port %i, but password invalid", m_iPort );
			nzblog( "nzbget request received on port %i, but password invalid", m_iPort );

			if( iSocket > -1 )
			{
			  close( iSocket );
			}
			
			continue;
		}

		// Info - connection received
		struct sockaddr_in PeerName;
		int iPeerNameLength = sizeof( PeerName );

		if( getpeername( iSocket, (struct sockaddr*)&PeerName, (socklen_t*) &iPeerNameLength) >= 0 )
		{
			info( "%s request received from %s", g_szMessageRequests[pMessageBase->m_iType], inet_ntoa( PeerName.sin_addr) );
			nzblog( "%s request received from %s", g_szMessageRequests[pMessageBase->m_iType], inet_ntoa( PeerName.sin_addr) );
		}
				
		int iExtraDataLength = iRequestReceived - pMessageBase->m_iSize;
		char* pExtraData = (iExtraDataLength > 0) ? ( (char*)&RequestBuffer + pMessageBase->m_iSize) : NULL;

		switch( pMessageBase->m_iType )
		{
			case NZBMessageRequest::eRequestDownload:
			{
				char* pRecvBuffer = NULL;
				int iBytesReceived = 0;

				SNZBQueueDownloadRequest* pDownloadRequest = (SNZBQueueDownloadRequest*) &RequestBuffer;

				pRecvBuffer = new char[ pDownloadRequest->m_iTrailingDataLength + 1 ];
				memset( pRecvBuffer, 0, pDownloadRequest->m_iTrailingDataLength + 1 ); // Clear the buffer

				if( pExtraData )
				{
					memcpy( pRecvBuffer, pExtraData, iExtraDataLength );
					iBytesReceived += iExtraDataLength;
#ifdef NZBGET_TEST_SEND_RECEIVE
					printf( "copying extra data %i\n", iBytesReceived );
#endif
				}

				char* pBuffer = pRecvBuffer + iBytesReceived;

				// Read from the socket until nothing remains
				do
				{
					iResult = m_pConnection->Recv( iSocket );

					// Did the recv succeed?
					if( iResult > 0 )
					{
#ifdef NZBGET_TEST_SEND_RECEIVE
                        // TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST
						// TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST 
						FILE* pFile = fopen( "transmitted.txt", "a+" );

						if( !pFile )
						{
							perror( "fopen" );
						}
						else
						{
//							fprintf( pFile, "\n---------------[ NEW RECEIVE ]-----------\n" );
							fprintf( pFile, "%s", m_pConnection->GetReadBuffer() );
						}
						fclose( pFile );
						// TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST
						// TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST 
#endif
                        memcpy( pBuffer, m_pConnection->GetReadBuffer(), iResult );

						iBytesReceived += iResult;
						pBuffer += iResult;

#ifdef NZBGET_TEST_SEND_RECEIVE
						printf( "BytesReceived %i, BytesReceivedTotal %i, TrailingDataLength %i\n", iResult, iBytesReceived, pDownloadRequest->m_iTrailingDataLength );
#endif

						if( iBytesReceived >= pDownloadRequest->m_iTrailingDataLength )
						{
							break;
						}
					}
					else if( iResult < 0 )
					{
						if( pRecvBuffer )
						{
							delete pRecvBuffer;
							pRecvBuffer = NULL;
						}
						
						break;;
					}
					else
					{
						break;
					}
				}
				while( iResult != 0 );

				if( pRecvBuffer )
				{
#ifdef NZBGET_TEST_SEND_RECEIVE
					// TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST
					// TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST 
					FILE* pFile = fopen( "received.txt", "a+" );

					if( !pFile )
					{
						perror( "fopen" );
					}
					else
					{
						fprintf( pFile, "\n---------------[ NEW RECEIVE ]-----------\n" );
						fprintf( pFile, "%s", pRecvBuffer );
						fclose( pFile );
					}

					printf( "iBytesReceived %i,pDownloadRequest->m_iTrailingDataLength %i\n",iBytesReceived, pDownloadRequest->m_iTrailingDataLength );

					// TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST 
					// TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST 
#endif
										
					NZBQueue* pQueue = m_pQueueCoordinator->CreateNZBQueueFromBuffer( pDownloadRequest->m_szFileName, pRecvBuffer, pDownloadRequest->m_iTrailingDataLength );

					if( pQueue )
					{
						info( "Request: Queue file %s", pDownloadRequest->m_szFileName );

						m_pQueueCoordinator->AddNZBQueueToDownloadQueue( pQueue, true );

						char tmp[1024];
						sprintf( tmp, "NZBFile %s added to queue", pDownloadRequest->m_szFileName );
						m_pConnection->Send( iSocket, tmp, strlen( tmp ) );
					}
					else
					{
						char tmp[1024];
						sprintf( tmp, "Download Request failed %s", pDownloadRequest->m_szFileName );
						m_pConnection->SendBuffer( iSocket, tmp, strlen( tmp ) );

#ifdef NZBGET_TEST_SEND_RECEIVE
                        // TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST
						// TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST 
						FILE* pFile = fopen( "badreceive.txt", "a+" );

						if( !pFile )
						{
							perror( "fopen" );
						}
						else
						{
							fprintf( pFile, "\n---------------[ NEW RECEIVE ]-----------\n" );
							fprintf( pFile, "%s", pRecvBuffer );
						}
						fclose( pFile );
						// TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST
						// TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST TEST
#endif
					}

					free( pRecvBuffer );
				}
				
				break;
			}
			case NZBMessageRequest::eRequestList:
			{

				// Make a data structure and copy all the elements of the list into it
				std::deque<NZBQueue*> pList = m_pQueueCoordinator->GetNZBQueueList();
						
				SNZBQueueListRequestAnswerEntry* pListAnswer = new SNZBQueueListRequestAnswerEntry[ pList.size() ];

				int iCount = 0;
				std::deque<NZBQueue*>::iterator it;
				for( it=pList.begin(); it!=pList.end(); it++ )
				{
					strcpy( pListAnswer[iCount].m_szFileName, (*it)->GetBaseFileName() );
					pListAnswer[iCount].m_iEntry					= iCount;
					pListAnswer[iCount].m_iOriginalNumberDownloads	= (*it)->GetOriginalNumberOfJobs();
					pListAnswer[iCount].m_iNumberDownloads			= (*it)->GetNumberOfJobs();
					iCount++;
				}

				//
				int iSize = sizeof( SNZBQueueListRequestAnswerEntry ) * pList.size();

				SNZBQueueListRequestAnswer ListRequestAnswer;
				InitMessageBase( &ListRequestAnswer.m_MessageBase, NZBMessageRequest::eRequestListAnswer, sizeof( ListRequestAnswer ) );
				ListRequestAnswer.m_iNrTrailingEntries	= iCount;
				ListRequestAnswer.m_fDownloadRate		= g_pNZBQueueCoordinator->CalcCurrentDownloadSpeed();

				// Send the request answer
				m_pConnection->Send( iSocket, (char*) &ListRequestAnswer, sizeof( ListRequestAnswer ) );

				// Send the data
				if( pList.size() > 0 )
				{
					m_pConnection->Send( iSocket, (char*) pListAnswer, iSize );
				}
				break;
			}
				
			case NZBMessageRequest::eRequestPauseUnpause:
			{
				SNZBQueuePauseUnpauseRequest* pPauseUnpauseRequest = (SNZBQueuePauseUnpauseRequest*) &RequestBuffer;

				GlobalControl::GetInstance()->SetPause( pPauseUnpauseRequest->m_bPause );

				g_pFrontend->Refresh();
				break;
			}
			case NZBMessageRequest::eRequestSetDownloadRate:
			{
				SNZBQueueSetDownloadRate* pSetDownloadRequest = (SNZBQueueSetDownloadRate*) &RequestBuffer;

				GlobalControl::GetInstance()->SetDownloadRate( pSetDownloadRequest->m_fDownloadRate );
				g_pFrontend->Refresh();
				break;
			}
			default:
				error( "NZB-Request not yet supported\n" );
				break;

		}

		// Close the socket
		if( iSocket > 0 )
		{
			close( iSocket );
		}

	}
}
