/*
 *  This file if part of nzbget
 *
 *  Copyright (C) 2004  Sven Henkel <sidddy@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Revision: 1.17 $
 * $Date: 2005/07/13 15:06:03 $
 *
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <libxml/xpath.h>
#include <libgen.h>
#include <stdlib.h>
#include <unistd.h>
#include <pwd.h>
#include <getopt.h>

#include "Options.h"
#include "global.h"
#include "GlobalControl.h"

#include "SingleServerPool.h"
#include "MultiServerPool.h"
#include "NewsServer.h"

extern float g_fDownloadRate;

static struct option long_options[] =
  {
    {"help", 0, 0, 'h'},
    {"tempdir", 1, 0, 't'},
    {"destdir", 1, 0, 'd'},
    {"outputmode", 1, 0, 'm'},
    {"appenddir", 1, 0, 'a'},
    {"configfile", 1, 0, 'c'},
    {"dupecheck", 1, 0, 'D'},
	{"renamebroken", 1, 0, 'b'},
//    {"downloadrate",1,0, 'r' },
    {"server",0,0,'s' },
    {"standalone",0,0,'n'},
	{"list",0,0,'L'},
	{"pause",0,0,'P'},
	{"unpause",0,0,'U'},
	{"rate",1,0,'R'},
	{"version",0,0,'v'},
	{"reloadqueue",0,0,'y'},
	{0, 0, 0, 0}
  };

//static char short_options[] = "a::c:d:D::hm:t:l:r:snv";
static char short_options[] = "a::c:d:D::hm:t:r:b::ysnvLPUR:";

Options::Options(int argc, char* argv[])
{
  // initialize options with default values

  this->initDefault();
  
  this->initOptFile(argc, argv);

  this->initCommandLine(argc, argv);

  this->checkOptions();

  this->initSpecials(argc, argv);

  this->initServers();

  //  this->dump();
  //exit(-1);
}

void Options::dump()
{
	for (uint i = 0; i < this->optEntries.size(); i++) 
	{
		printf("%s = \"%s\"\n", this->optEntries[i].name,		this->optEntries[i].value);
	}
}

void Options::initDefault()
{
	setOption( OPTION_TEMPDIR, ".");
	setOption( OPTION_DESTDIR, ".");
	setOption( OPTION_OUTPUT, "curses");
	setOption( OPTION_APPENDDIR, "no");
	setOption( OPTION_CONFIGFILE, "~/.nzbget/nzbget.cfg");
	setOption( OPTION_DUPECHECK, "yes");
	setOption( OPTION_STANDALONE,"no");
	setOption( OPTION_SERVER, "no" );
	setOption( OPTION_DOWNLOADRATE, "na" );
	setOption( OPTION_BINPATH, "/usr/bin/nzbget" );
	setOption( OPTION_SERVERIP, "localhost" );
	setOption( OPTION_SERVERPASSWORD, "tegbzn" );
	setOption( OPTION_SERVERQUEUEDIR, "~/.nzbget/queue/" );
	setOption( OPTION_SERVERPORT, "6789" );
	setOption( OPTION_USEQUEUEDIR, "yes" );
	setOption( OPTION_DOWNLOADTIMEOUT, "60" );
	setOption( OPTION_RENAMEBROKEN, "yes" );
	setOption( OPTION_RELOADQUEUE, "ask" );
	setOption( OPTION_USEBROKENLOG, "no" );
}

void Options::initOptFile(int argc, char* argv[])
{
	while(true)
	{
		int option_index  = 0;
		int c;
		
		c=getopt_long(argc, argv, short_options, long_options,&option_index);

		if (c==-1) break;
		
		switch (c)
		{
			case 'c':
				this->setOption(OPTION_CONFIGFILE, optarg);
				break;
		}
	}

  //TODO: Automatic xml-option-handling

  xmlDocPtr cfgDoc = xmlParseFile(this->getOption(OPTION_CONFIGFILE, NULL));
  if (!cfgDoc)
    {
      printf("FATAL ERROR: Configuration file %s not found or broken! Aborting.\n", this->getOption(OPTION_CONFIGFILE, "N/A"));
      exit(-1);
    }

	if (char* v = this->getXMLOption(cfgDoc, OPTION_TEMPDIR, NULL))
		this->setOption(OPTION_TEMPDIR, v);
	
	if (char* v = this->getXMLOption(cfgDoc, OPTION_DESTDIR, NULL))
		this->setOption(OPTION_DESTDIR, v);
	
	if (char* v = this->getXMLOption(cfgDoc, OPTION_OUTPUT, NULL))
		this->setOption(OPTION_OUTPUT, v);
	
	if (char* v = this->getXMLOption(cfgDoc, OPTION_APPENDDIR, NULL))
		this->setOption(OPTION_APPENDDIR, v);
	
	if (char* v = this->getXMLOption(cfgDoc, OPTION_DUPECHECK, NULL))
		this->setOption(OPTION_DUPECHECK, v);
		
	if( char* v = getXMLOption( cfgDoc, OPTION_DOWNLOADRATE, NULL ) )
	{
		setOption( OPTION_DOWNLOADRATE, v );
		GlobalControl::GetInstance()->SetDownloadRate( atof( v ) );
	}
	if( char* v = getXMLOption( cfgDoc, OPTION_STANDALONE, NULL ) )
	{
		setOption( OPTION_STANDALONE, v );
	}
	if( char* v = getXMLOption( cfgDoc, OPTION_BINPATH, NULL ) )
	{
		setOption( OPTION_BINPATH, v );
	}

	if( char* v = getXMLOption( cfgDoc, OPTION_SERVERIP, NULL ) )
	{
		setOption( OPTION_SERVERIP, v );
	}
	if( char* v = getXMLOption( cfgDoc, OPTION_SERVERPORT, NULL ) )
	{
		setOption( OPTION_SERVERPORT, v );
	}
	
	if( char* v = getXMLOption( cfgDoc, OPTION_SERVERPASSWORD, NULL ) )
	{
		setOption( OPTION_SERVERPASSWORD, v );
	}
	if( char* v= getXMLOption( cfgDoc, OPTION_USEQUEUEDIR, NULL ) )
	{
		setOption( OPTION_USEQUEUEDIR, v );
	}
	if( char* v = getXMLOption( cfgDoc, OPTION_DOWNLOADTIMEOUT, NULL ) )
	{
		setOption( OPTION_DOWNLOADTIMEOUT, v );
	}
	if( char* v = getXMLOption( cfgDoc, OPTION_RENAMEBROKEN, NULL ) )
	{
		setOption( OPTION_RENAMEBROKEN, v );
	}
	if( char* v = getXMLOption( cfgDoc, OPTION_RELOADQUEUE, NULL ) )
	{
		setOption( OPTION_RELOADQUEUE, v );
	}

	if( char* v = getXMLOption( cfgDoc, OPTION_INFOTARGET, NULL ) )
	{
		if( !strcasecmp( v, "screen" ) )
		{
			GlobalControl::GetInstance()->SetInfoTarget( GlobalControl::eScreen );
		}
		else if( !strcasecmp( v, "log" ) )
		{
			GlobalControl::GetInstance()->SetInfoTarget( GlobalControl::eLog );
		}
		else if( !strcasecmp( v, "both" ) )
		{
			GlobalControl::GetInstance()->SetInfoTarget( GlobalControl::eBoth );
		}
		else if( !strcasecmp( v, "none" ) )
		{
			GlobalControl::GetInstance()->SetInfoTarget( GlobalControl::eNone );
		}
		else
		{
			printf( "Warning: wrong parameter for .cfg file (%s)", v );
		}
	}
	if( char* v = getXMLOption( cfgDoc, OPTION_WARNINGTARGET, NULL ) )
	{
		if( !strcasecmp( v, "screen" ) )
		{
			GlobalControl::GetInstance()->SetWarningTarget( GlobalControl::eScreen );
		}
		else if( !strcasecmp( v, "log" ) )
		{
			GlobalControl::GetInstance()->SetWarningTarget( GlobalControl::eLog );
		}
		else if( !strcasecmp( v, "both" ) )
		{
			GlobalControl::GetInstance()->SetWarningTarget( GlobalControl::eBoth );
		}
		else if( !strcasecmp( v, "none" ) )
		{
			GlobalControl::GetInstance()->SetWarningTarget( GlobalControl::eNone );
		}
		else
		{
			printf( "Warning: wrong parameter for .cfg file (%s)", v );
		}
	}
	if( char* v = getXMLOption( cfgDoc, OPTION_ERRORTARGET, NULL ) )
	{
		if( !strcasecmp( v, "screen" ) )
		{
			GlobalControl::GetInstance()->SetErrorTarget( GlobalControl::eScreen );
		}
		else if( !strcasecmp( v, "log" ) )
		{
			GlobalControl::GetInstance()->SetErrorTarget( GlobalControl::eLog );
		}
		else if( !strcasecmp( v, "both" ) )
		{
			GlobalControl::GetInstance()->SetErrorTarget( GlobalControl::eBoth );
		}
		else if( !strcasecmp( v, "none" ) )
		{
			GlobalControl::GetInstance()->SetErrorTarget( GlobalControl::eNone );
		}
		else
		{
			printf( "Warning: wrong parameter for .cfg file (%s)", v );
		}
	}
	if( char* v = getXMLOption( cfgDoc, OPTION_USEBROKENLOG, NULL ) )
	{
		if( !strcasecmp( v, "yes" ) )
		{
			GlobalControl::GetInstance()->SetCreateBrokenFileLog( true );
		}
		else
		{
			GlobalControl::GetInstance()->SetCreateBrokenFileLog( false );
		}
	}
}


void Options::initCommandLine(int argc, char* argv[])
{
	m_eNZBOperation = eNoOperation; // default

	// reset getopt
	optind = 1;

	while (1) 
	{
      int option_index  = 0;
      int c;
      
      c=getopt_long( argc, argv, short_options, long_options, &option_index );
      
      if (c==-1) break;
      
		switch (c) 
		{
			case 'a':
				if (optarg)
				{
					if ((!strcasecmp(optarg, "yes")) || (!strcasecmp(optarg, "+")))
					{
						this->setOption(OPTION_APPENDDIR, "yes");
					}
					else if ((!strcasecmp(optarg, "no")) || (!strcasecmp(optarg, "-")))
					{
						this->setOption(OPTION_APPENDDIR, "no");
					}
					else
					{
						printf("FATAL ERROR: Could not parse value of option 'a'\n");
						exit(-1);
					}
				}
				else
				{
					this->setOption(OPTION_APPENDDIR, "yes");
				}
				break;
			case 'd':
				this->setOption(OPTION_DESTDIR, optarg);
				break;
			case 'D':
				if (optarg)
				{
					if ((!strcasecmp(optarg, "yes")) || (!strcasecmp(optarg, "+")))
					{
						this->setOption(OPTION_DUPECHECK, "yes");
					}
					else if ((!strcasecmp(optarg, "no")) || (!strcasecmp(optarg, "-")))
					{
						this->setOption(OPTION_DUPECHECK, "no");
					}
					else
					{
						printf("FATAL ERROR: Could not parse value of option 'a'\n");
						exit(-1);
					}
				}
				else
				{
					this->setOption(OPTION_DUPECHECK, "yes");
				}
				break;
			case 'h':
				this->printUsage(argv[0]);
				exit(0);
				break;
			case 'm':
				this->setOption(OPTION_OUTPUT, optarg);
				break;
			case 't':
				this->setOption(OPTION_TEMPDIR, optarg);
				break;
			case 'r':
				this->setOption(OPTION_DOWNLOADRATE, optarg);
				break;
			case 'b':
				if (optarg)
				{
					if ((!strcasecmp(optarg, "yes")) || (!strcasecmp(optarg, "+")))
					{
						this->setOption(OPTION_RENAMEBROKEN, "yes");
					}
					else if ((!strcasecmp(optarg, "no")) || (!strcasecmp(optarg, "-")))
					{
						this->setOption(OPTION_RENAMEBROKEN, "no");
					}
					else
					{
						printf("FATAL ERROR: Could not parse value of option 'b'\n");
						exit(-1);
					}
				}
				else
				{
					this->setOption(OPTION_RENAMEBROKEN, "yes");
				}
				break;
			case 's':
				setOption( OPTION_SERVER, "yes" );
				break;
			case 'n':
				setOption( OPTION_STANDALONE, "yes" );
				break;
			case 'L':
				m_eNZBOperation = eClientRequestList;
				break;
			case 'P':
				m_eNZBOperation = eClientRequestPause;
				break;
			case 'U':
				m_eNZBOperation = eClientRequestUnpause;
				break;
			case 'R':
				m_eNZBOperation = eClientRequestSetRate;
				setOption(OPTION_CLIENTSETRATE, optarg);
				break;
			case 'y':
				setOption( OPTION_RELOADQUEUE, "yes" );
				break;
			case 'v':
			        printf ("nzbget version: %s\n", VERSION);
				exit(1);
				break;
		}
	}
}

void Options::printUsage(char* com)
{
  printf("Usage: %s [options] <nzb-file>\n"
	 "Options:\n"
	 "  -a, --appenddir [<+|-|yes|no>]  Switch for appending the post's\n"
	 "                                  name after destination-directory\n"
	 "                                  Works only if the name is given in\n"
	 "                                  the filename of the nzb-file.\n"
	 "                                  Default: yes\n"
	 "  -c, --configfile <file>         Filename of configation-file\n"
	 "  -d, --destdir <dir>             Directory for downloaded files\n"
	 "  -D, --dupecheck [<+|-|yes|no>]  Switch to enble/disable nzbget's\n"
	 "                                  dupecheck. Default: yes\n"
	 "  -h, --help                      Print this help-message\n"
	 "  -m, --outputmode <mode>         Output-mode, where mode is:\n"
	 "                                  colored, curses, logable\n"
	 "  -t, --tempdir <dir>             Directory for temporary files\n"
	 "  -r, --downloadrate <number>     Sets the maximum download rate (bandwidth throttle)\n"
	 "  -b, --renamebroken [<yes|no>]   Rename broken files. Default: yes\n"
	 "  -s, --server                    Starts nzbget as a server\n"
	 "  -y, --reloadqueue				Forces reloading of previous queue upon startup\n"
	 "  -n, --standalone                Starts nzbget in the original mode\n"
	 "  -L, --list                      Requests list of downloads from server\n"
	 "  -P, --pause                     Pause downloading on the server\n"
	 "  -U, --unpause                   Unpause downloading on the server\n"
	 "  -R, --rate                      Set the download rate on the server\n"
	 "  -v, --version                   Print version and exit\n"
	 "",
	 com);
}

void Options::initSpecials(int argc, char* argv[])
{
	if( m_eNZBOperation == eNoOperation )
	{
		if (optind >= argc)
		{
			printf( "No nzb-file given - starting server-mode\n" );
			setOption( OPTION_SERVER, "yes" );
		}
		else
		{
			// Check if the file-name is a relative path or an absolute path
			// If the path starts with '/' its an absolute, else relative
			const char* szFileName = argv[optind];
			
			if( szFileName[0] == '/' )
			{
				setOption( OPTION_NZBFILE, szFileName );
			}
			else
			{
				// TEST
				char szFileNameWithPath[1024];
				getcwd( szFileNameWithPath, 1024 );
				strcat( szFileNameWithPath, "/" );
				strcat( szFileNameWithPath, szFileName );
				setOption( OPTION_NZBFILE, szFileNameWithPath );
			}
				
			setOption( OPTION_SERVER, "no" );
		}
	}

    this->setOption(OPTION_REALDESTDIR, this->getOption(OPTION_DESTDIR, "."));
}


void Options::setOption(char* optname, const char* value)
{
  if (this->getOption(optname, NULL) != NULL)
    this->delOption(optname);

  OptEntry newo;
  newo.name = strdup(optname);

  if ((value) && (value[0] == '~') && (value[1] == '/'))
    {
      // expand home-dir

      char* home = getenv("HOME");
      if (!home)
	{
	  struct passwd *pw = getpwuid(getuid());
	  if (pw)
	    home = pw->pw_dir;
	}

      if (!home)
	{
	  printf("FATAL ERROR: Unable to determine home-directory. Aborting!\n");
	  exit(-1);
	}

      char* newvalue = (char*) malloc(strlen(value) + strlen(home) + 10);

      if (home[strlen(home)-1] == '/')
	sprintf(newvalue, "%s%s", home, value+2);
      else
	sprintf(newvalue, "%s/%s", home, value+2);      

      newo.value = strdup(newvalue);
      
      free(newvalue);
    }
  else
    newo.value = strdup(value);
  optEntries.push_back(newo);
}

void Options::delOption(char* optname)
{
  for (uint i = 0; i < this->optEntries.size(); i++) 
    {
      if (!strcasecmp(this->optEntries[i].name, optname))
	{
	  this->optEntries.erase(this->optEntries.begin() + i);
	  return;
	}
    }
}

char* Options::getOption(char* optname, char* defaultValue)
{
  if (!optname)
    return NULL;

  for (uint i = 0; i < this->optEntries.size(); i++) 
    {
      if (!strcasecmp(this->optEntries[i].name, optname))
	{
	  return this->optEntries[i].value;
	}
    }
  return defaultValue;
}

char* Options::getXMLOption(xmlDocPtr optdoc, char* optname, char* defaultValue)
{

  char* res = defaultValue;

  xmlXPathContextPtr context;
  xmlXPathObjectPtr result;

  context = xmlXPathNewContext(optdoc);

  char tmp[1024];
  snprintf(tmp, 1024, "//option[@name='%s']", optname);
  
  result = xmlXPathEvalExpression((xmlChar *) tmp, context);

  if ((result) && (result->nodesetval) && (result->nodesetval->nodeTab) &&
      (result->nodesetval->nodeTab[0]) && (result->nodesetval->nodeTab[0]->children) &&
      (result->nodesetval->nodeTab[0]->children->content))

    res = (char*)result->nodesetval->nodeTab[0]->children->content;

  if (result)
    xmlXPathFreeObject(result);
  xmlXPathFreeContext(context);

  return res;
}


void Options::initServers()
{

  xmlDocPtr optdoc = xmlParseFile(this->getOption(OPTION_CONFIGFILE, NULL));
  if (!optdoc)
    {
      printf("FATAL ERROR: Configuration file %s not found or broken! Aborting.\n", this->getOption(OPTION_CONFIGFILE, "N/A"));
      exit(-1);
    }

  for (int level=0; level < 100; level++)
    {

      xmlXPathContextPtr context;
      xmlXPathObjectPtr result;

      context = xmlXPathNewContext(optdoc);

      char tmp[1024];
      snprintf(tmp, 1024, "//server[@level='%d']", level);
      
      result = xmlXPathEvalExpression((xmlChar *) tmp, context);
      
      if ((result) && (result->nodesetval) && (result->nodesetval->nodeTab) &&
          (result->nodesetval->nodeTab[0]))
        {
          xmlNodePtr servernode = result->nodesetval->nodeTab[0];

          char* host = NULL;
          int port = 119;
          char* user = NULL;
          char* pass = NULL;
          int conns = 1;

          xmlNodePtr child = servernode->children;

          while (child)
            {
              if (child->children)
                {
                  if (!strcmp((char*)child->name, "host"))
                    host = (char*)child->children->content;
                  if (!strcmp((char*)child->name, "port"))
                    port = atoi((char*)child->children->content);
                  if (!strcmp((char*)child->name, "user"))
                    user = (char*)child->children->content;
                  if (!strcmp((char*)child->name, "pass"))
                    pass = (char*)child->children->content;
                  if (!strcmp((char*)child->name, "connections"))
                    conns = atoi((char*)child->children->content);
                }
              child = child->next;
            }

          if (host)
            {
              NewsServer* serv = new NewsServer(host, port, user, pass);
              SingleServerPool* pool = new SingleServerPool(serv, conns);
              MultiServerPool::GetInstance()->AddServerPool(pool);
            }
        }
      else
        level = 1000;
      
      if (result)
        xmlXPathFreeObject(result);
      xmlXPathFreeContext(context);
    }
}

void Options::checkOptions()
{
  char* v;
  v = this->getOption(OPTION_DESTDIR, NULL);
  if (!v)
    abort("FATAL ERROR: Destination-directory undefined. Check options.\n");
  if (v[0] == '~')
    abort("FATAL ERROR: Destination-directory invalid. Home-expansion currently no fully supported.\n");

  v = this->getOption(OPTION_TEMPDIR, NULL);
  if (!v)
    abort("FATAL ERROR: Temp-directory undefined. Check options.\n");
  if (v[0] == '~')
    abort("FATAL ERROR: Temp-directory invalid. Home-expansion currently no fully supported.\n");

  v = this->getOption(OPTION_APPENDDIR, NULL);
  if (!v)
    abort("FATAL ERROR: Appenddir undefined. Check options.\n");
  if (strcmp(v, "yes") && strcmp(v, "no"))
    abort("FATAL ERROR: Appenddir-option invalid. Check options.\n");

  v = this->getOption(OPTION_OUTPUT, NULL);
  if (!v)
    abort("FATAL ERROR: Outputmode undefined. Check options.\n");
  if (strcmp(v, "colored") && strcmp(v, "logable") && strcmp(v, "curses") )
    abort("FATAL ERROR: Outputmode invalid. Check options.\n");

  v = this->getOption(OPTION_DUPECHECK, NULL);
  if (!v)
    abort("FATAL ERROR: Dupecheck-option undefined. Check options.\n");
  if (strcmp(v, "yes") && strcmp(v, "no"))
    abort("FATAL ERROR: Dupecheck-option invalid. Check options.\n");
}
