/*
 *  This file if part of nzbget
 *
 *  Copyright (C) 2004  Sven Henkel <sidddy@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Revision: 1.13 $
 * $Date: 2005/07/13 15:06:03 $
 *
 */

#ifndef NZBOPTIONS_H
#define NZBOPTIONS_H

#include <libxml/parser.h>
#include <vector>

#define OPTION_DESTDIR			"destinationdir"
#define OPTION_TEMPDIR			"tempdir"
#define OPTION_APPENDDIR		"appendpostnametodir"
#define OPTION_OUTPUT			"outputmode"
#define OPTION_CONFIGFILE		"configfile"
#define OPTION_DUPECHECK		"dupecheck"
#define OPTION_NZBFILE			"nzbfile"
#define OPTION_REALDESTDIR		"realdestinationdir"
#define OPTION_SERVER			"server"
#define OPTION_DOWNLOADRATE		"downloadrate"
#define OPTION_RENAMEBROKEN     "renamebroken"
#define OPTION_STANDALONE		"standalone"
#define OPTION_BINPATH			"binpath"
#define OPTION_SERVERIP			"serverip"
#define OPTION_SERVERPORT		"serverport"
#define OPTION_SERVERPASSWORD	"serverpassword"
#define OPTION_SERVERQUEUEDIR	"serverqueuedir"
#define OPTION_DOWNLOADTIMEOUT  "downloadtimeout"
#define OPTION_USEQUEUEDIR		"usequeuedir"
#define OPTION_CLIENTLIST       "clientlist"
#define OPTION_CLIENTPAUSE      "clientpause"
#define OPTION_CLIENTUNPAUSE    "clientunpause"
#define OPTION_CLIENTSETRATE    "clientsetrate"
#define OPTION_RELOADQUEUE		"reloadqueue"
#define OPTION_INFOTARGET		"info"
#define OPTION_WARNINGTARGET	"warning"
#define OPTION_ERRORTARGET		"error"
#define OPTION_USEBROKENLOG		"createbrokenfilelog"

struct OptEntry
{
    char* name;
    char* value;
};

class Options
{
public:
	enum EOperation
	{
		eNoOperation,
		eClientRequestDownload,
		eClientRequestList,
		eClientRequestPause,
		eClientRequestUnpause,
		eClientRequestSetRate
	};

	EOperation              		m_eNZBOperation;
	char*							add_dir;
	std::vector< struct OptEntry >	optEntries;
public:
    
    								Options(int argc, char* argv[]);
    								~Options();

    char*							getOption(char* optname, char* defaultValue);
    void							setOption(char* optname, const char* value);
	void							delOption(char* optname);

protected:
    void							initDefault();
    void							initOptFile(int argc, char* argv[]);
    void							initCommandLine(int argc, char* argv[]);
    void							initSpecials(int argc, char* argv[]);
    void							initServers();
    void							checkOptions();
    void							printUsage(char* com);
    void							dump();
    char*							getXMLOption(xmlDocPtr cfgDoc, char* optname, char* defaultValue);
};

#endif
