/*
 *  This file if part of nzbget
 *
 *  Copyright (C) 2004  Sven Henkel <sidddy@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Revision: 1.14 $
 * $Date: 2005/07/12 09:37:48 $
 *
 */

/*
 *  
 * m_Semaphore Patch by Florian Penzkofer <f.penzkofer@sent.com>
 * The queue of mutexes that was used did not work for every
 * implementation of POSIX threads. Now a m_Semaphore is used.
 *   
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <unistd.h>
#include <semaphore.h>

#include "SingleServerPool.h"
#include "global.h"

SingleServerPool::SingleServerPool( NewsServer* pNewsServer, int iMax )
{
	debug("SingleServerPool::SingleServerPool()\n");
	m_pNewsServer	= pNewsServer;
	m_iMax			= iMax;
	m_iId			= 0;
	m_Connections.clear();
	m_FreeConnections.clear();
	m_Queue.clear();
	
	pthread_mutex_init( &m_MutexFree, NULL );
	sem_init( &m_Semaphore, 0, iMax );

	for( int i = 0; i < iMax; i++ )
	{
		m_Connections.push_back( new NNTPConnection( m_pNewsServer ) );
		m_FreeConnections.push_back( i );
	}
}

int SingleServerPool::GetID()
{
	debug("SingleServerPool::GetID() : Assigning ID %d\n",m_iId+1);
	return ++m_iId;
}

NNTPConnection* SingleServerPool::GetConnection()
{
	int connection_nr = -1;
		
	// decrease m_Semaphore counter or block
	sem_wait( &m_Semaphore );
	
	pthread_mutex_lock( &m_MutexFree );

	if( m_FreeConnections.size() > 0 )
	{
		// free connection found, take it!
		connection_nr = m_FreeConnections.front();
		m_FreeConnections.pop_front();
	}
	
	pthread_mutex_unlock( &m_MutexFree );
		
	if (connection_nr == -1)
	{
		fprintf(stderr, "SingleServerPool: serious error, no free connection found, but there should be one.\n");
		return NULL;
	}
	
	return m_Connections[connection_nr];
}

void SingleServerPool::FreeConnection( NNTPConnection* pConnection )
{
    // give back free connection
	for( int i = 0; i < m_iMax; i++ )
    {
		if( m_Connections[i] == pConnection )
		{
			debug("SingleServerPool::freeConnection(NNTPConnection*) : Connection %d freed\n",i);
 		
			pthread_mutex_lock( &m_MutexFree );
			m_FreeConnections.push_back( i );
			pthread_mutex_unlock( &m_MutexFree );
			sem_post( &m_Semaphore );
		}
    }
}
