/*
 *  This file if part of nzbget
 *
 *  Copyright (C) 2004  Sven Henkel <sidddy@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Revision: 1.5 $
 * $Date: 2005/07/06 18:34:34 $
 *
 */

#ifndef THREAD_H
#define THREAD_H

#include <pthread.h>

/**
 * Generic class for all classes running a second POSIX thread.
 * Just take care that the run() method is overloaded and the 
 * start() routine is called.
 */
class Thread
{
	pthread_t 				m_Thread;
	pthread_attr_t 			m_Attr;
	bool 					m_bRunning;		
public:
							Thread();
	virtual 				~Thread();
	
	virtual void 			Start();
	virtual void 			Stop();
	
	bool 					IsRunning()	const { return m_bRunning; }
	void 					SetRunning( bool bOnOff ) { m_bRunning = bOnOff; }
	pthread_t 				GetThread() { return m_Thread; }
	void 					SetThread(pthread_t pThread) { m_Thread = pThread; }
	
protected:
	
	virtual void 			Run() {}; // Virtual function - override in derivatives
	
	/**
	* Static function calling the run() method of the specified
	* Thread object.
	* @param obj Thread object whose run() method should be executed as
	*        2nd thread.
	* @return NULL
	*/
	static void *thread_handler(void *obj);
	
private:
};

#endif
