/*
 *  This file if part of nzbget
 *
 *  Copyright (C) 2004  Sven Henkel <sidddy@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Revision: 1.14 $
 * $Date: 2005/07/13 15:06:03 $
 *
 */


#include "Frontend.h"
#include "Options.h"
#include "global.h"
#include "config.h"
#include "GlobalControl.h"

#include <stdarg.h>

// Variables
Frontend* g_pFrontend	= NULL;
Options* g_pOptions		= NULL;

#define USE_NZBLOG

#ifdef DEBUG
void debug(char* msg, ...)
{
	char tmp2[1024];
	
	va_list ap;
	va_start(ap, msg);
	vsnprintf(tmp2, 1024, msg, ap);
	va_end(ap);
	
	if( g_pFrontend )
	{
		g_pFrontend->FrontendDebug(tmp2);
	}
	else
	{
		printf( "%s\n", tmp2 );
	}
}
#endif

void error(char* msg, ...)
{
	char tmp2[1024];
	
	va_list ap;
	va_start(ap, msg);
	vsnprintf(tmp2, 1024, msg, ap);
	va_end(ap);
	
	GlobalControl::EMessageTarget eMessageTarget = GlobalControl::GetInstance()->GetErrorTarget();
	if( eMessageTarget == GlobalControl::eScreen || eMessageTarget == GlobalControl::eBoth )
	{
		if( g_pFrontend )
		{
			g_pFrontend->FrontendError(tmp2);
		}
		else
		{
			printf( "%s\n", tmp2 );
		}
	}
	if( eMessageTarget == GlobalControl::eLog || eMessageTarget == GlobalControl::eBoth )
	{
		nzblog( "[ERROR] %s", tmp2 );
	}
}

void warn(char* msg, ...)
{
	char tmp2[1024];
	
	va_list ap;
	va_start(ap, msg);
	vsnprintf(tmp2, 1024, msg, ap);
	va_end(ap);
	
	GlobalControl::EMessageTarget eMessageTarget = GlobalControl::GetInstance()->GetWarningTarget();
	if( eMessageTarget == GlobalControl::eScreen || eMessageTarget == GlobalControl::eBoth )
	{
		if( g_pFrontend )
		{
			g_pFrontend->FrontendWarn(tmp2);
		}
		else
		{
			printf( "%s\n", tmp2 );
		}
	}
	if( eMessageTarget == GlobalControl::eLog || eMessageTarget == GlobalControl::eBoth )
	{
		nzblog( "[WARNING] %s", tmp2 );
	}
}

void info(char* msg, ...)
{
	char tmp2[1024];
	
	va_list ap;
	va_start(ap, msg);
	vsnprintf(tmp2, 1024, msg, ap);
	va_end(ap);

	GlobalControl::EMessageTarget eMessageTarget = GlobalControl::GetInstance()->GetInfoTarget();
	if( eMessageTarget == GlobalControl::eScreen || eMessageTarget == GlobalControl::eBoth )
	{
		if( g_pFrontend )
		{
			g_pFrontend->FrontendInfo(tmp2);
		}
		else
		{
			printf( "%s\n", tmp2 );
		}
	}
	if( eMessageTarget == GlobalControl::eLog || eMessageTarget == GlobalControl::eBoth )
	{
		nzblog( "[INFO] %s", tmp2 );
	}
}

void abort(char* msg, ...)
{
	char tmp2[1024];
	
	va_list ap;
	va_start(ap, msg);
	vsnprintf(tmp2, 1024, msg, ap);
	va_end(ap);
	
	printf("%s", tmp2);

	nzblog( tmp2 );
	
	exit(-1);
}
                                           
void nzblog(char* msg, ...)
{
#ifdef USE_NZBLOG
	char tmp2[1024];
	
	va_list ap;
	va_start(ap, msg);
	vsnprintf(tmp2, 1024, msg, ap);
	va_end(ap);

	char szTime[512];
	time_t rawtime;
	time ( &rawtime );
	ctime_r(&rawtime, szTime);
	szTime[ strlen(szTime) -1 ] = 0;

	char szDir[1024];
	strcpy( szDir, g_pOptions->getOption(OPTION_REALDESTDIR, ".") );
	strcat( szDir, "/nzbget.log" );
	FILE* file = fopen( szDir, "a+" );
	fprintf( file, "[%s] %s\n", szTime, tmp2 );
	fclose( file );
#endif
}
